package 
{
	import adobe.utils.CustomActions;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.geom.Point;
	import flash.utils.Timer;
	import flash.utils.getTimer;
	
	/**
	$(CBI)* ...
	$(CBI)* @author Guchan Alkan
	$(CBI)*/
	public class Main extends Sprite 
	{
		private var node:Node; // node object
		public var nodeArray:Array = new Array(); // an array that holds the nodes
		private var idCounter:int = 0; // total number of the nodes
		public var startNodeId:int = 0; // start node id
		public var finishNodeId:int = 0; // finish node id
		public var stepCounter:int = 1; // total number steps to go to the finish nodes
		private var currentNodeId:int = 0; // current node id which is in the frontier list
		private var parentId:int = 0; // parent node id
		private var deltaTime:int = 0; // the time passed for the calculation in miliseconds
		private var userInterface:UserInterface = new UserInterface(); // user interface
		private var frontierCount:int = 0; // number of nodes which are in the frontier list
		private var timer:Timer = new Timer(500); // timer for step by step simulation
		
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			
			Utilities.main = this;
			
			// grid is drawn and added to the stage
			drawGrid();
			
			// user interface is added to the stage
			addChild(userInterface);
			userInterface.x = 400;
			userInterface.setTextField("Simulation Mode: Instant");
			
			stage.addEventListener(Event.ENTER_FRAME, onLoop);
			timer.addEventListener(TimerEvent.TIMER, onTimer);
		}
		public function drawGrid():void
		{
			// these nested for loops draws the grid
			for (var i:int = 0; i < Utilities.rowCount; i++)
			{
				for (var j:int = 0; j < Utilities.columnCount; j++)
				{
					node = new Node(idCounter, i, j, 48);
					node.x = j * 50;
					node.y = i * 50;
					addChild(node);
					idCounter++;
					nodeArray.push(node);
				}
			}
		}
		private function onLoop(e:Event):void
		{
			deltaTime = getTimer();
		}
		private function onTimer(e:TimerEvent):void
		{
			iterateNextStep();
		}
		public function findPath():void
		{
			for (var i:int = 0; i < nodeArray.length; i++)
			{
				nodeArray[i].resetNode();
			}
			currentNodeId = startNodeId;
			stepCounter = 1;
			parentId = 0;
			calculateHeuristicDistance(nodeArray[finishNodeId].position);
			iterateNextStep();
		}
		private function iterateNextStep():void
		{
			if (currentNodeId != finishNodeId)
			{
				nodeArray[currentNodeId].addNeighboursToFrontier();
				if (frontierCount != 0)
				{
					if (Utilities.instantSimulation)
					{
						iterateNextStep();
					}
					else
					{
						timer.start();
					}
				}
				else
				{
					trace("path is blocked");
					userInterface.setTextField("path is blocked");
					timer.stop();
				}
			}
			else
			{
				parentId = finishNodeId;
				drawPath();
				timer.stop();
				
				var simulationMode:String;
				var pathLength:String;
				if (Utilities.instantSimulation)
				{
					simulationMode = "Simulation Mode: Instant";
					var timeInMs:String = "time taken: " + (getTimer() - deltaTime) + " ms";
					pathLength = "path length: " + nodeArray[finishNodeId].gCost;
					
					userInterface.setTextField(simulationMode + "\n\n" + timeInMs + "\n" + pathLength);
				}
				else
				{
					simulationMode = "Simulation Mode: Step by Step";
					pathLength = "path length: " + nodeArray[finishNodeId].gCost;
					
					userInterface.setTextField(simulationMode + "\n\n" + pathLength);
				}
			}
		}
		public function drawPath():void
		{
			while (parentId != startNodeId)
			{
				nodeArray[parentId].makePath();
				parentId = nodeArray[parentId].parentNodeId;
			}
			nodeArray[startNodeId].makePath();
		}
		public function chooseBestFrontier():void
		{
			var lowestFCost:int = 1000;
			frontierCount = 0;
			for (var i:int = 0; i < nodeArray.length; i++)
			{
				if (nodeArray[i].isFrontier)
				{
					frontierCount++;
					if (nodeArray[i].fCost < lowestFCost)
					{
						lowestFCost = nodeArray[i].fCost;
						currentNodeId = i;
					}
				}
			}
		}
		public function calculateHeuristicDistance(position:Point):void // calculates the heuristic cost for each node
		{
			for (var i:int = 0; i < nodeArray.length; i++)
			{
				nodeArray[i].calculateDistance(position);
			}
		}
		
	}
	
}