package  
{
	import adobe.utils.ProductManager;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.geom.ColorTransform;
	import flash.geom.Point;
	import flash.text.TextField;
	import flash.text.TextFormat;
	/**
	$(CBI)* ...
	$(CBI)* @author Guchan Alkan
	$(CBI)*/
	public class Node extends Sprite
	{
		private var nodeText:TextField; // text showing node properties
		private var frontSprite:Sprite = new Sprite(); // graphical element
		private var id:int = 0; // node id
		private var neighbourArray:Array = new Array(); // array that holds node's neighbours
		public var visited:Boolean = false; // true if the node is visited
		public var position:Point = new Point(); // node's position on the cartesian coordinate system
		public var enabled:Boolean = true; // true if node is not visited or an obstacle vs.
		public var isFrontier:Boolean = false; // true if the node is in the frontier array
		public var parentNodeId:int = 0; // node's parent node's id
		
		public var hCost:int = 0; // The estimated movement cost from this node to the target node
		public var gCost:int = 0; // The movement cost from starting node to this node
		public var fCost:int = 0; // gCost + hCost
		
		public function Node(id:int, i:int, j:int, size:int) 
		{
			this.id = id;
			position.x = i;
			position.y = j;
			
			this.graphics.beginFill(0x999999);
			this.graphics.drawRect(0, 0, size, size);
			this.graphics.endFill();
			
			nodeText = Utilities.myTextField();
			nodeText.text = "H: " + hCost + "\nG: " + gCost + "\nF: " + fCost;
			addChild(nodeText);
			
			frontSprite.graphics.beginFill(0xFF0055);
			frontSprite.graphics.drawRect(0, 0, size, size);
			frontSprite.graphics.endFill();
			frontSprite.alpha = 0;
			frontSprite.buttonMode = true;
			addChild(frontSprite);
			frontSprite.addEventListener(MouseEvent.CLICK, onClick);
			
			calculateNeighbours();
		}
		private function onClick(e:MouseEvent):void
		{
			switch (Utilities.nodeType) 
			{
				case 0:
					beStart();
				break;
				case 1:
					beFinish();
				break;
				case 2:
					beObstacle();
				break;
			}
		}
		private function beStart():void
		{
			Utilities.main.nodeArray[Utilities.main.startNodeId].resetNode();
			Utilities.main.startNodeId = id;
			enabled = true;
			frontSprite.transform.colorTransform = new ColorTransform(0, 0, 0, 1, 102, 204, 153, 0);
			frontSprite.alpha = 0.5;
		}
		private function beFinish():void
		{
			Utilities.main.nodeArray[Utilities.main.finishNodeId].resetNode();
			Utilities.main.finishNodeId = id;
			enabled = true;
			frontSprite.transform.colorTransform = new ColorTransform(0, 0, 0, 1, 255, 204, 0, 0);
			frontSprite.alpha = 0.5;
		}
		private function beObstacle():void
		{
			if (enabled)
			{
				enabled = false;
				frontSprite.transform.colorTransform = new ColorTransform(0, 0, 0, 1, 0, 0, 0, 0);
				frontSprite.alpha = 1;
			}
			else
			{
				enabled = true;
				frontSprite.alpha = 0;
			}
		}
		public function resetNode():void
		{
			if (enabled)
			{
				frontSprite.alpha = 0;
			}
			visited = false;
			isFrontier = false;
			parentNodeId = 0;
			hCost = 0;
			gCost = 0;
			fCost = 0;
		}
		public function makePath():void
		{
			frontSprite.transform.colorTransform = new ColorTransform(0, 0, 0, 1, 255, 0, 85, 0);
			frontSprite.alpha = .5;
		}
		public function calculateDistance(position:Point):void
		{
			hCost = Math.abs(this.position.x - position.x) + Math.abs(this.position.y - position.y);
			nodeText.text = "H: " + hCost + "\nG: " + gCost + "\nF: " + fCost;
		}
		public function addNeighboursToFrontier():void
		{
			frontSprite.transform.colorTransform = new ColorTransform(0, 0, 0, 1, 0, 102, 255, 0);
			frontSprite.alpha = 0.5;
			isFrontier = false;
			visited = true;
			for (var i:int = 0; i < neighbourArray.length; i++)
			{
				if (!Utilities.main.nodeArray[neighbourArray[i]].visited && Utilities.main.nodeArray[neighbourArray[i]].enabled)
				{
					Utilities.main.nodeArray[neighbourArray[i]].parentNodeId = id;
					Utilities.main.nodeArray[neighbourArray[i]].gCost = this.gCost + 1;
					Utilities.main.nodeArray[neighbourArray[i]].calculateFCost();
					Utilities.main.nodeArray[neighbourArray[i]].isFrontier = true;
					//trace(id);
				}
			}
			Utilities.main.chooseBestFrontier();
		}
		// calculates the F cost, remember F = G + H
		public function calculateFCost():void
		{
			fCost = gCost + hCost;
			nodeText.text = "H: " + hCost + "\nG: " + gCost + "\nF: " + fCost;
		}
		// specifies the neighbours of the node
		private function calculateNeighbours():void
		{
			var rowCount:int = Utilities.rowCount;
			var columnCount:int = Utilities.columnCount;
			var gridSize:int = rowCount * columnCount;
			
			//if (id >= 11 && id % 10 != 0) neighbourArray.push(id - 11);
			if (id >= columnCount) neighbourArray.push(id - columnCount);
			//if (id >= 9 && (id + 1) % 10 != 0) neighbourArray.push(id - 9);
			if (id >= 1 && id % columnCount != 0) neighbourArray.push(id - 1);
			if (id <= gridSize - 2 && (id + 1) % columnCount != 0) neighbourArray.push(id + 1);
			//if (id <= 90 && id % 10 != 0) neighbourArray.push(id + 9);
			if (id < gridSize - columnCount) neighbourArray.push(id + columnCount);
			//if (id <= 88 && (id + 1) % 10 != 0) neighbourArray.push(id + 11);
		}
	}

}